/*
 * Copyright (c) 2006, Your Corporation. All Rights Reserved.
 */

package net.rmi.utils;

import gui.In;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * DocJava, Inc.
 * http://www.docjava.com
 * Programmer: Francisco Castellanos & Douglas Lyon
 * Date: March 14, 2006
 * net.rmi.utils.RmiRegistryUtils
 * Is a Parametric singleton pattern that enables the creation of one
 * and only one instance of the registry given a set of ports.
 * Once the registry exists on a given port, it is not created again.
 */
public final class ParametricSingletonRmiRegistry {

    private static Hashtable registry = new Hashtable(100);
    // use singleton pattern and
    // prevent instantiation of RmiRegistryUtils.

    private ParametricSingletonRmiRegistry() {
    }
    // the only instance of the Registry held by the RmiRegistryUtils


    public static void listNames(Integer port) {
        if (!isInCacheAndRunning(port))
            return;
        Registry r = (Registry) registry.get(port);
        System.out.println("registry on port: " + port);
        System.out.println("registry: " + r);
        System.out.println("names: ");
        try {
            print(r.list());
        } catch (RemoteException e) {
            In.message(e);
        }
        System.out.println("------------------------");
    }

    public static void listNamesForAllRegistries() {

        Enumeration enm = registry.keys();
        Integer port;
        try {
            while (enm.hasMoreElements()) {
                port = (Integer) enm.nextElement();
                if (!isInCacheAndRunning(port))
                    continue;
                Registry r = (Registry) registry.get(port);
                System.out.println("registry on port: " + port);
                System.out.println("registry: " + r);
                System.out.println("names: ");
                print(r.list());
                System.out.println("------------------------");
            }
        } catch (Exception e) {
            In.message(e);
        }
    }

    public static void print(Object o[]) {
        for (int i = 0; i < o.length; i++)
            System.out.println(o[i]);
    }

    /**
     * Checks whether a Registry has been started on this host
     * in the specified port.
     * If not, it tries to start one.
     * when it fails to start a Registry, it exits the program
     * with exit code 1.
     */
    public static void restart(int port) {
        if (isInCacheAndRunning(port))
            return;
        try {
            startRegistry(port);
        } catch (RemoteException e) {
            In.message(e);

        }
    }

    /**
     * Overloaded method for default port.
     * Checks whether a Registry has been started on this host.
     * If not, it tries to start one.
     * when it fails to start a Registry, it exits the program
     * with exit code 1.
     */
    public static void restart() {
        Integer port = new Integer(Registry.REGISTRY_PORT);
        if (isInCacheAndRunning(port))
            return;
        try {
            startRegistry(port);
        } catch (RemoteException e) {
            In.message(e);
        }
    }

    /**
     * @param port
     * @return true if registry is in cache and running
     */
    private static boolean isInCacheAndRunning(Integer port) {
        if (!isRegistryRunning(port.intValue())) return false;
        if (!isRegistryInCache(port.intValue())) return false;
        return true;
    }

    /**
     * @param port
     * @return true if we are able to locate a local registry on <code>port</code>.
     */
    private static boolean isRegistryRunning(int port) {
        try {
            Registry r = LocateRegistry.getRegistry(port);
            if (r != null) return true;
        } catch (RemoteException e) {
            return false;
        }
        return false;
    }

    /**
     * @param port
     * @return return true if registry is in cache
     */
    private static boolean isRegistryInCache(int port) {
        if (registry.get(new Integer(port)) == null) return false;
        return true;
    }

    /**
     * Restart the registry, if needed.
     * Return the only instance of the registry for consistent global
     * using by making use of the Singleton Design Pattern
     *
     * @return the internally held registry instance.
     */
    public static Registry getRegistry(Integer port) {
        restart(port);
        return (Registry) registry.get(port);
    }

    /**
     * Overloaded method for default port.
     * Restart the registry, if needed.
     * Return the only instance of the registry for consistent global
     * using by making use of the Singleton Design Pattern
     *
     * @return the internally held registry instance.
     */
    public static Registry getRegistry() {
        Integer port = new Integer(Registry.REGISTRY_PORT);
        restart(port);
        return (Registry) registry.get(port);
    }

    private static void startRegistry(Integer port) throws RemoteException {
        Registry r;
        try {
            r = LocateRegistry.createRegistry(port.intValue());
            registry.put(port, r);
            //r = new sun.rmi.registry.RegistryImpl
            ///        (port.intValue());
            // registry.put(port, r);
        } catch (Exception e) {
            //e.printStackTrace();
            r = LocateRegistry.getRegistry(port.intValue());
            registry.put(port, r);
        }
    }

    public static void main(String args[]) throws RemoteException {
        //restart(port);
        restart(5001);
        restart(5002);
        restart(5002);
        Registry r = LocateRegistry.getRegistry(5002);
        System.out.println("registry="+r);
        listNamesForAllRegistries();
    }
}

